function [dzdx] = ddx(z,x)

%ddx(z,x): Differentiates matrix z with respect to x using a
% centred difference approximation (uncentred at the edges)
% Use: dzdx=ddx(z,x)
% x can be a matrix, in which case x-values increase across rows

[ny nx]=size(z);
[m n]=size(x);
if m==1
  xg=meshgrid(x,1:ny);
else
  xg=x;
end

dx(:,2:(nx-1))=xg(:,3:nx)-xg(:,1:(nx-2));

dx(:,1)=xg(:,3)-xg(:,1);
dx(:,nx)=xg(:,nx)-xg(:,(nx-2));

dzdx(:,2:(nx-1))=(z(:,3:nx)-z(:,1:(nx-2)))./dx(:,2:(nx-1));

dzdx(:,1)=(4*z(:,2)-3*z(:,1)-z(:,3))./dx(:,1);
dzdx(:,nx)=-(4*z(:,(nx-1))-3*z(:,nx)-z(:,(nx-2)))./dx(:,1);