#include "mex.h"
/*

	Solves poisson's equation to get the streamlines
	psi=mfish(vort, dx, dz, psi_top, psi_bot);
	Danny Chapman 19/09/2000

 */

void mfish(double *psi, double *vort, double dx, double dz, psi_top, psi_bot, int rows, int cols);

void mexFunction(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{
	double *psi, *vort;
	double dx, dz, psi_top, psi_bot;
	int rows, cols;

	/* Check arguments */
	if (nrhs!=5)
		mexErrMsgTxt("Three inputs required");
/* 	if (nlhs!=1) */
/* 		mexErrMsgTxt("One output required"); */
	
	/* Create a pointer to the input matrices, vu and Z */
	vort=mxGetPr(prhs[0]);
	rows=mxGetM(prhs[0]);
	cols=mxGetN(prhs[0]);

	/* Get the scalar input, dz */
	dx=mxGetScalar(prhs[1]);
	dz=mxGetScalar(prhs[2]);

	psi_top=mxGetScalar(prhs[3]);
	psi_bot=mxGetScalar(prhs[4]);
	
	/* Set the ouput pointer to the output matrix */
	plhs[0]=mxCreateDoubleMatrix(rows,cols, mxREAL);

	/* Create a pointer to a copy of the ouput matrix*/
	psi=mxGetPr(plhs[0]);

	/* Call the shear subroutine */
	mfish(psi,vort,dx,dy,psi_top,psi_bot,rows,cols);
}

void mfish(double *psi, double *vort, double dx, double dz, psi_top, psi_bot, int rows, int cols)
{
	int i, j, count;
	double dx2, dz2, dt, dtdx2, dtdz2;

	dx2=dx*dx;
	dz2=dz*dz;
	dt = (dx2/4 <  dz2/4 ? dx2/4 : dz2/4);
	dtdx2=dt/dx2;
	dtdz2=dt/dz2;

	/* boundary conditions */
	for (i=0; i < nx ; i++) {
		psi(i,0)=psi_bot;
		psi(i,nz-1)=psi_top;
	}
	/*
left and right are going to be given by vertical velocity=0..., and
we will update psi at the boundary
*/

	for (count=0 ; count < 1000 ; count++) {
		for (i=1; i < nx-1 ; i++) {
			for (j=1; j < ny-1 ; j++) {
				psi(i,j)=psi(i,j)+dtdx2*(psi(i+1,j)+psi(i-1,j)-2*psi(i,j))+dtdz2*(psi(i,j+1)+psi(i,j-1)-2*psi(i,j))-dt*vort(i,j);
			}
		}
	}
	for (j=1; j < ny-1 ; j++) {
		psi(0,j)=psi(0,j)+dtdx2*(psi(1,j)-psi(0,j))+dtdz2*(psi(0,j+1)+psi(0,j-1)-2*psi(0,j))-dt*vort(0,j);
		psi(nx-1,j)=psi(nx-1,j)+dtdx2*(-psi(nx-1,j)+psi(nx-2,j))+dtdz2*(psi(nx-1,j+1)+psi(nx-1,j-1)-2*psi(nx-1,j))-dt*vort(nx-1,j);
	}

	/* that's it */
}
















	int i, j, ii, jj, jtop, jbot;
	double *temp1, *temp2, *temp3, *slope, *offset, *vu, *Z;
	mxArray *lhs[2], *rhs[3], *vup[1], *Zp[1];
	int flip=0;
	double thisz, ztop, vtop, zbot, vbot, dv;

	rhs[0]=mxCreateDoubleMatrix(1,cols, mxREAL);
	temp1=mxGetPr(rhs[0]); /* z vals */
	rhs[1]=mxCreateDoubleMatrix(1,cols, mxREAL);
	temp2=mxGetPr(rhs[1]); /* velocities */
	rhs[2]=mxCreateDoubleMatrix(1,1, mxREAL);
	temp3=mxGetPr(rhs[2]); /* degree of fit */
	temp3[0]=1;

	lhs[0]=mxCreateDoubleMatrix(1,1, mxREAL);
	slope=mxGetPr(lhs[0]); /* slope of fit */
	lhs[1]=mxCreateDoubleMatrix(1,1, mxREAL);
	offset=mxGetPr(lhs[1]); /* offset of fit */

	vup[0]=mxCreateDoubleMatrix(rows,cols, mxREAL);
	vu=mxGetPr(vup[0]);
	Zp[0]=mxCreateDoubleMatrix(rows,cols, mxREAL);
	Z=mxGetPr(Zp[0]);

	/* vu must be already unfolded and zapped */

/*  	printf("rows = %d, cols = %d \n", rows, cols);  */

	if (Z_in[0+0*rows] > Z_in[0+(cols-1)*rows]) {
		flip=1;		
		/* flip vu and z*/
		for (i=0; i<rows; i++) {
			for (j=0; j<cols; j++) {
				vu[i+j*rows]=vu_in[i+(cols-j-1)*rows];
				Z[i+j*rows]=Z_in[i+(cols-j-1)*rows];
			}
		}
	} else { /* don't flip */
		for (i=0; i<rows; i++) {
			for (j=0; j<cols; j++) {
				vu[i+j*rows]=vu_in[i+j*rows];
				Z[i+j*rows]=Z_in[i+j*rows];
			}
		}
	}

	/* Start properly */

	for (i=0; i<rows; i++) {
/* 		printf("%d ",i); */
		for (j=0; j<cols; j++) {
			/*find the points dz/2 above and below the current point */
			thisz=Z[i+j*rows];
			/*search upwards...*/
			ztop=mxGetNaN();
			vtop=mxGetNaN();
			jtop=-1; jbot=-1;
			for (jj=j; jj<cols; jj++) {
				if (Z[i+jj*rows] > (thisz+0.5*dz) ) {
/* 					ztop=Z[i+jj*rows]; */
/* 					vtop=vu[i+jj*rows]; */
/* 					vtop=interp(Z[i+(jj-1)*rows],ztop,thisz+0.5*dz,vu[i+(jj-1)*rows],vu[i+jj*rows]); */
/* 					ztop=thisz+0.5*dz; */
					jtop=jj;
					break;
				}
			}
			/*search downwards...*/
			zbot=mxGetNaN();
			vbot=mxGetNaN();
			for (jj=j; jj>=0; jj--) {
				if (Z[i+jj*rows] < (thisz-0.5*dz) ) {
/* 					zbot=Z[i+jj*rows]; */
/* 					vbot=vu[i+jj*rows]; */
/* 					vbot=interp(Z[i+(jj+1)*rows],zbot,thisz-0.5*dz,vu[i+(jj+1)*rows],vu[i+jj*rows]); */
/* 					zbot=thisz-0.5*dz; */
					jbot=jj;
					break;
				}
			}
			/* this just uses the difference from top to bottom...*/
			/* shear[i+j*rows]=(vtop-vbot)/(ztop-zbot); */
			/* fit a straight line between top and bottom */
			if ((jtop != -1) && (jbot != -1)) {
				for (jj=jbot; jj<=jtop ; jj++) {
					temp2[jj-jbot]=vu[i+jj*rows];
					temp1[jj-jbot]=Z[i+jj*rows];
				}
				mxSetN(rhs[0],1+jtop-jbot);
				mxSetN(rhs[1],1+jtop-jbot);
				/* POLYFIT(X,Y,N) finds the coefficients of a polynomial P(X)
					 of degree N that fits the data, P(X(I))~=Y(I), in a
					 least-squares sense. */
/* 				mexCallMATLAB(2,lhs,3,rhs,"polyfit"); */
/* 				slope=mxGetPr(lhs[0]);  */
/* slope of fit */
/* 				printf("slope=%g\t",slope[0]);  */
				fit(temp1,temp2,1+jtop-jbot,slope);
/* 				printf("slope2=%g\n",slope[0]);  */
				shear[i+j*rows]=slope[0];

/* 				if ((i+j*rows) == (40+(cols-90)*rows)) */
/* 					shear[i+j*rows]=-1000; */
				
			} else
				shear[i+j*rows]=mxGetNaN();
		}
	}

	/*flip shear if necessary, use vu as a temporary array*/
	if (Z_in[0+0*rows] > Z_in[0+(cols-1)*rows]) {
		flip=1;		
		/* flip shear and store in vu */
		for (i=0; i<rows; i++) {
			for (j=0; j<cols; j++) {
				vu[i+j*rows]=shear[i+(cols-j-1)*rows];
			}
		}
		/*...and copy the result back into shear */
		for (i=0; i<rows; i++) {
			for (j=0; j<cols; j++) {
				shear[i+j*rows]=vu[i+j*rows];
			}
		}
	}

	mxDestroyArray(lhs[0]);
	mxDestroyArray(lhs[1]);
	mxDestroyArray(rhs[0]);
	mxDestroyArray(rhs[1]);
	mxDestroyArray(rhs[2]);
	mxDestroyArray(vup[0]);
	mxDestroyArray(Zp[0]);

}

/* Given a set of data points x[1..ndata],y[1..ndata] with individual
   standard deviations sig[1..ndata], t them to a straight line y = a
   + bx by minimizing 2 . Returned are a,b and their respective
   probable uncertainties siga and sigb, the chi-square chi2, andthe
   goodness-of- t probability q (that the t would have 2 this large or
   larger). If mwt=0 on input, then the standard deviations are
   assumed to be unavailable: q is returned as 1.0 and the
   normalization of chi2 is to unit standard deviation on all
   points. */

static double sqrarg; 
#define SQR(a) ((sqrarg=(a)) == 0.0 ? 0.0 : sqrarg*sqrarg) 

#include <math.h> 
void fit(double x[], double y[], int ndata, double *b) 
{
	/*	double gammq(double a, double x); */
	int i; 
	double wt,t,sxoss,sx=0.0,sy=0.0,st2=0.0,ss,sigdat; 
	*b=0.0; 
	for (i=0;i<ndata;i++) { 
		sx += x[i]; sy +=	y[i]; 
	} 
	ss=ndata; 
	sxoss=sx/ss;
	for (i=0;i<ndata;i++) { 
		t=x[i]-sxoss; 
		st2 += t*t;
		*b += t*y[i];
	}
	*b /= st2;
	/*	*a=(sy-sx*(*b))/ss;*/
}


